@@ -110,7 +110,7 @@ namespace Beagle {
 
 		[XmlAttribute ("Uri")]
 		public string UriString {
-			get { return uri.ToString (); }
+			get { return UriFu.UriToSerializableString (uri); }
 			set { uri = UriFu.UriStringToUri (value); }
 		}
 
@@ -122,7 +122,7 @@ namespace Beagle {
 
 		[XmlAttribute ("ContentUri")]
 		public string ContentUriString {
-			get { return ContentUri.ToString (); }
+			get { return UriFu.UriToSerializableString (ContentUri); }
 			set { contentUri = UriFu.UriStringToUri (value); } 
 		}
 
@@ -134,10 +134,15 @@ namespace Beagle {
 		
 		[XmlAttribute ("HotContentUri")]
 		public string HotContentUriString {
-			get { return HotContentUri != null ? HotContentUri.ToString () : ""; }
+			get { return HotContentUri != null ? UriFu.UriToSerializableString (HotContentUri) : ""; }
 			set { hotContentUri = (value != "") ? new Uri (value) : null; }
 		}
 
+		[XmlIgnore]
+		public Uri DisplayUri {
+			get { return uri.Scheme == "uid" ? ContentUri : Uri; }
+		}
+
 		[XmlAttribute]
 		public bool DeleteContent {
 			get { return deleteContent; }
@@ -223,7 +228,8 @@ namespace Beagle {
 		{
 			StringBuilder sb = new StringBuilder ();
 			foreach (Property prop in Properties) {
-				if (keywords ? prop.IsKeyword : ! prop.IsKeyword) {
+				if (prop.IsSearched 
+				    && keywords ? prop.IsKeyword : ! prop.IsKeyword) {
 					if (sb.Length > 0)
 						sb.Append (" ");
 					sb.Append (prop.Value);
