@@ -73,11 +73,13 @@ namespace Best {
 
 		void CreateWindow (string query)
 		{
-			Title = "Bleeding-Edge Search Tool";
+			Title = Best.DefaultWindowTitle;
 
 			DeleteEvent += new DeleteEventHandler (this.DoDelete);
+			MapEvent += new MapEventHandler (MapIt);
+			UnmapEvent += new UnmapEventHandler (UnmapIt);
 
-			Icon = Images.GetPixbuf ("dog.png");
+			Icon = Images.GetPixbuf ("best.png");
 
 			Widget content = CreateContents ();
 
@@ -123,6 +125,18 @@ namespace Best {
 						    Gdk.ModifierType.ControlMask,
 						    Gtk.AccelFlags.Visible);
 
+			// Previous Page (PageUp)
+			global_keys.AddAccelerator (new EventHandler (this.PageBackHandler),
+						    (uint) Gdk.Key.Page_Up, 
+						    0,
+						    Gtk.AccelFlags.Visible);
+
+			// Next Page (PageDown)
+			global_keys.AddAccelerator (new EventHandler (this.PageForwardHandler),
+						    (uint) Gdk.Key.Page_Down, 
+						    0,
+						    Gtk.AccelFlags.Visible);
+
 			DBusisms.BeagleDown += OnBeagleDown;
 
 			UpdatePage ();
@@ -134,6 +148,34 @@ namespace Best {
 
 		//////////////////////////
 
+		int posX = 0, posY = 0;
+
+		public new void Show ()
+		{
+			base.Show ();	
+			
+			Move (posX, posY);
+			Present ();
+		}
+
+		public new void Hide ()
+		{
+			// FIXME: Hack, why does Hide () gets invoked twice, the second time with (0,0) as window position?
+			
+			int new_posX = 0, new_posY = 0;
+
+			GetPosition (out new_posX, out new_posY);
+
+			if (new_posX != 0 &&  new_posY != 0) {
+				posX = new_posX;
+				posY = new_posY;
+			}
+
+			base.Hide ();
+		}
+
+		//////////////////////////
+
 		Query query = null;
 		string hit_type = null;
 		
@@ -158,6 +200,18 @@ namespace Best {
 			Hide ();
 		}
 
+		public bool WindowIsVisible;
+
+		private void MapIt (object o, MapEventArgs args)
+		{			
+			WindowIsVisible = true;
+		}
+
+		private void UnmapIt (object o, UnmapEventArgs args)
+		{
+			WindowIsVisible = false;
+		}
+
 		private void HideWindowHandler (object o, EventArgs args)
 		{
 			Hide ();
@@ -181,18 +235,34 @@ namespace Best {
 		
 		//////////////////////////
 
-		private void DoForwardClicked (object o, EventArgs args)
+		private void PageForward ()
 		{
+			if (!root.HitCollection.CanPageForward)
+				return;
+
 			root.HitCollection.PageForward ();
 			UpdatePage ();
 		}
 
-		private void DoBackClicked (object o, EventArgs args)
+		private void PageBack ()
 		{
+			if (!root.HitCollection.CanPageBack)
+				return;
+
 			root.HitCollection.PageBack ();
 			UpdatePage ();
 		}
 
+		private void PageForwardHandler (object o, EventArgs args)
+		{
+			PageForward ();
+		}
+
+		private void PageBackHandler (object o, EventArgs args)
+		{
+			PageBack ();
+		}
+
 		//////////////////////////
 
 		private Gnome.Entry entry;
@@ -289,14 +359,14 @@ namespace Best {
 			forward_button = StockButton ("gtk-go-forward", 
 						      "Show More Results");
 			forward_button.Show ();
-			forward_button.Clicked += new EventHandler (DoForwardClicked);
+			forward_button.Clicked += new EventHandler (PageForwardHandler);
 			pager.PackEnd (forward_button, false, false, 3);
 
 			back_button = StockButton ("gtk-go-back",
 						   "Show Previous Results");
 			back_button.Show ();
 
-			back_button.Clicked += new EventHandler (DoBackClicked);
+			back_button.Clicked += new EventHandler (PageBackHandler);
 			pager.PackEnd (back_button, false, false, 3);
 
 			pager.Show ();
@@ -362,25 +432,8 @@ namespace Best {
 		
 		private void DoSearch (object o, EventArgs args)
 		{
-			try {
+			if (entry.GtkEntry.Text != null && entry.GtkEntry.Text != "")
 				Search (entry.GtkEntry.Text);
-			}
-			catch (Exception e)
-			{
-				delayedQuery = entry.GtkEntry.Text;
-				DBusisms.BeagleUpAgain += QueueDelayedQuery;
-
-				if (e.ToString ().IndexOf ("'com.novell.Beagle'") != -1) {
-					root.Error ("The query for <i>" + entry.GtkEntry.Text + "</i> failed." +
-						    "<br>The likely cause is that the beagle daemon isn't running.");
-					root.OfferDaemonRestart = true;
-				} else if (e.ToString().IndexOf ("Unable to determine the address") != -1) {
-					root.Error ("The query for <i>" + entry.GtkEntry.Text + "</i> failed.<br>" +
-						    "The session bus isn't running.  See http://beaglewiki.org/index.php/Installing%20Beagle for information on setting up a session bus.");
-				} else
-					root.Error ("The query for <i>" + entry.GtkEntry.Text + "</i> failed with error:<br><br>" + e);
-			}
-			UpdatePage ();
 		}
 
 		private string lastType = null;
@@ -402,8 +455,7 @@ namespace Best {
 			//	Search (entry.GtkEntry.Text);
 			//	lastType = this.hit_type;
 			//}
-			
-			UpdatePage ();
+			//UpdatePage ();
 		}
 
 		//////////////////////////
@@ -459,30 +511,47 @@ namespace Best {
 
 		private void Search (String searchString)
 		{
-			StoreSearch (searchString);
-			entry.GtkEntry.Text = searchString;
-			if (query != null) {
-				query.Cancel ();
-				DetachQuery ();
+			try {
+				StoreSearch (searchString);
+				entry.GtkEntry.Text = searchString;
+
+				if (query != null) {
+					query.Cancel ();
+					DetachQuery ();
+				}
+
+				query = Factory.NewQuery ();
+				query.AddDomain (QueryDomain.Neighborhood);
+
+				// FIXME: Disable non-local searching for now.
+				//query.AddDomain (QueryDomain.Global);
+
+				query.AddText (searchString);
+				root.SetSource (hit_type);
+				
+				AttachQuery ();
+				
+				root.Query = query;
+				root.Start ();
+				
+				SetBusy (true);
+				query.Start ();
+			} catch (Exception e) {
+				delayedQuery = entry.GtkEntry.Text;
+				DBusisms.BeagleUpAgain += QueueDelayedQuery;
+
+				if (e.ToString ().IndexOf ("com.novell.Beagle") != -1) {
+					root.Error ("The query for <i>" + searchString + "</i> failed." +
+						    "<br>The likely cause is that the beagle daemon isn't running.");
+					root.OfferDaemonRestart = true;
+				} else if (e.ToString().IndexOf ("Unable to determine the address") != -1) {
+					root.Error ("The query for <i>" + searchString + "</i> failed.<br>" +
+						    "The session bus isn't running.  See http://beaglewiki.org/index.php/Installing%20Beagle for information on setting up a session bus.");
+				} else
+					root.Error ("The query for <i>" + searchString + "</i> failed with error:<br><br>" + e);
 			}
 
-			query = Factory.NewQuery ();
-			
-			query.AddDomain (QueryDomain.Neighborhood);
-
-			// Disable non-local searching for now.
-			//query.AddDomain (QueryDomain.Global);
-
-			query.AddText (searchString);
-			root.SetSource (hit_type);
-
-			AttachQuery ();
-			
-			root.Query = query;
-			root.Start ();
-
-			SetBusy (true);
-			query.Start ();
+			UpdatePage ();
 		}
 
 		ArrayList recentSearches;
