@@ -43,9 +43,22 @@ namespace Beagle.Filters {
 		
 		public FilterOpenOffice () 
 		{
+			// OO 1.0 mime types
 			AddSupportedMimeType ("application/vnd.sun.xml.writer");
-			AddSupportedMimeType ("application/vnd.sun.xml.impress");
+			AddSupportedMimeType ("application/vnd.sun.xml.writer.template");
 			AddSupportedMimeType ("application/vnd.sun.xml.calc");
+			AddSupportedMimeType ("application/vnd.sun.xml.calc.template");
+			AddSupportedMimeType ("application/vnd.sun.xml.impress");
+			AddSupportedMimeType ("application/vnd.sun.xml.impress.template");
+
+			// OO 2.0 mime types
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.text");
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.text-template");
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.spreadsheet");
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.spreadsheet-template");
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.presentation");
+			AddSupportedMimeType ("application/vnd.oasis.opendocument.presentation-template");
+
 			SnippetMode = true;
 			odtFormat = false;
 		}
@@ -460,11 +473,13 @@ namespace Beagle.Filters {
 			hotStyles = new Hashtable ();
 			try {
 				zip = new ZipFile (info.FullName);
-				if (MimeType == "application/vnd.oasis.opendocument.text")
+
+				if (MimeType.StartsWith ("application/vnd.oasis.opendocument."))
 					odtFormat = true;
+
 			} catch (Exception e) {
-				Logger.Log.Error ("Caught exception inside FilterOpenOffice.DoOpen");
-				Logger.Log.Error (e); 
+				Logger.Log.Error ("Unable to open {0}.  Probably an invalid OpenOffice document.");
+				Finished ();
 			}
 		}
 
