@@ -31,6 +31,7 @@ using System.Collections;
 using System.IO;
 using System.Runtime.InteropServices;
 using System.Text;
+using System.Text.RegularExpressions;
 using System.Threading;
 
 using Mono.Posix;
@@ -39,7 +40,7 @@ namespace Beagle.Util {
 
 	public class Inotify {
 
-		public delegate void Handler (int wd, string path, string subitem, EventType type, uint cookie);
+		public delegate void Handler (int wd, string path, string subitem, string srcpath, EventType type);
 
 		public static event Handler Event;
 
@@ -66,6 +67,9 @@ namespace Beagle.Util {
 			All            = 0xffffffff
 		}
 
+		// Events that we want internally, even if the handlers do not
+		static private EventType base_mask =  EventType.Ignored | EventType.MovedFrom | EventType.MovedTo;
+
 		/////////////////////////////////////////////////////////////////////////////////////
 
 		static private Logger log;
@@ -123,7 +127,7 @@ namespace Beagle.Util {
 
 			dev_inotify = Syscall.open ("/dev/inotify", OpenFlags.O_RDONLY);
 			if (dev_inotify == -1)
-				Logger.Log.Debug ("Could not open /dev/inotify");
+				Logger.Log.Warn ("Could not open /dev/inotify");
 		}
 
 		static public bool Enabled {
@@ -140,12 +144,32 @@ namespace Beagle.Util {
 
 			public EventType FilterMask;
 			public EventType FilterSeen;
+
+			public ArrayList Children;
+			public Watched   Parent;
 		}
 
 		static Hashtable watched_by_wd = new Hashtable ();
 		static Hashtable watched_by_path = new Hashtable ();
 		static Watched   last_watched = null;
 
+		private class PendingMove {
+			public Watched   Watch;
+			public string    SrcName;
+			public DateTime  Time;
+			public uint      Cookie;
+
+			public PendingMove (Watched watched, string srcname, DateTime time, uint cookie) {
+				Watch = watched;
+				SrcName = srcname;
+				Time = time;
+				Cookie = cookie;
+			}
+		}
+
+		static Hashtable cookie_hash = new Hashtable ();
+		static ArrayList cookie_list = new ArrayList ();
+
 		static public int WatchCount {
 			get { return watched_by_wd.Count; }
 		}
@@ -163,9 +187,9 @@ namespace Beagle.Util {
 		{
 			lock (watched_by_wd) {
 				Watched watched;
-				if (last_watched != null && last_watched.Wd == wd) {
+				if (last_watched != null && last_watched.Wd == wd)
 					watched = last_watched;
-				} else {
+				else {
 					watched = watched_by_wd [wd] as Watched;
 					if (watched != null)
 						last_watched = watched;
@@ -185,6 +209,8 @@ namespace Beagle.Util {
 		{
 			if (last_watched == watched)
 				last_watched = null;
+			if (watched.Parent != null)
+				watched.Parent.Children.Remove (watched);
 			watched_by_wd.Remove (watched.Wd);
 			watched_by_path.Remove (watched.Path);
 		}
@@ -203,7 +229,6 @@ namespace Beagle.Util {
 				throw new IOException (path);
 
 			lock (watched_by_wd) {
-				
 				Watched watched;
 
 				watched = watched_by_path [path] as Watched;
@@ -213,15 +238,12 @@ namespace Beagle.Util {
 					Forget (watched);
 				}
 
-				EventType internal_mask = mask;
-				internal_mask |= EventType.Ignored;
-
-				wd = inotify_glue_watch (dev_inotify, path, internal_mask);
+				wd = inotify_glue_watch (dev_inotify, path, mask | base_mask);
 				if (wd < 0) {
 					string msg = String.Format ("Attempt to watch {0} failed!", path);
 					throw new Exception (msg);
 				}
-				
+
 				watched = new Watched ();
 				watched.Wd = wd;
 				watched.Path = path;
@@ -231,6 +253,13 @@ namespace Beagle.Util {
 				watched.FilterMask = initial_filter;
 				watched.FilterSeen = 0;
 
+				watched.Children = new ArrayList ();
+
+				DirectoryInfo dir = new DirectoryInfo (path);
+				watched.Parent = watched_by_path [dir.Parent.ToString ()] as Watched;
+				if (watched.Parent != null)
+					watched.Parent.Children.Add (watched);
+
 				watched_by_wd [watched.Wd] = watched;
 				watched_by_path [watched.Path] = watched;
 			}
@@ -248,9 +277,8 @@ namespace Beagle.Util {
 			EventType seen = 0;
 
 			path = Path.GetFullPath (path);
-			
-			lock (watched_by_wd) {
 
+			lock (watched_by_wd) {
 				Watched watched;
 				watched = watched_by_path [path] as Watched;
 
@@ -258,7 +286,7 @@ namespace Beagle.Util {
 				watched.FilterMask = mask;
 				watched.FilterSeen = 0;
 			}
-			
+
 			return seen;
 		}
 
@@ -381,16 +409,72 @@ namespace Beagle.Util {
 			}
 		}
 
-		// FIXME: If we move a directory that is being watched, the
-		// watch is preserved, as are the watches on any
-		// subdirectories.  This means that our cached value from
-		// WatchedInfo.Path will no longer be accurate.  We need to
-		// trap MovedFrom and MovedTo, check if the thing being moved
-		// is a directory, and then and update our internal data
-		// structures accordingly.
-
 		static public bool Verbose = false;
 
+		// Update the watched_by_path hash and the path stored inside the watch
+		// in response to a move event.
+		static private void MoveWatch (Watched watch, string name)
+		{
+			watched_by_path.Remove (watch.Path);
+			watch.Path = name;
+			watched_by_path [watch.Path] = watch;
+
+			if (Verbose)
+				Console.WriteLine ("*** inotify: Moved Watch to {0}", watch.Path);
+		}
+
+		// A directory we are watching has moved.  We need to fix up its path, and the path of
+		// all of its subdirectories, their subdirectories, and so on.
+		static private void HandleMove (string srcpath, string dstpath)
+		{
+			Watched start = watched_by_path [srcpath] as Watched;	// not the same as src!
+			if (start == null) {
+				Console.WriteLine ("Lookup failed for {0}", srcpath);
+				return;
+			}
+
+			// Queue our starting point, then walk its subdirectories, invoking MoveWatch() on
+			// each, repeating for their subdirectories.  The relationship between start, child
+			// and dstpath is fickle and important.
+			Queue queue = new Queue();
+			queue.Enqueue (start);
+			do {
+				Watched target = queue.Dequeue () as Watched;
+				for (int i = 0; i < target.Children.Count; i++) {
+					Watched child = target.Children[i] as Watched;
+					string name = Path.Combine (dstpath, child.Path.Substring (start.Path.Length + 1));
+					MoveWatch (child, name);
+					queue.Enqueue (child);
+				}
+			} while (queue.Count > 0);
+
+			// Ultimately, fixup the original watch, too
+			MoveWatch (start, dstpath);
+		}
+
+		static private void SendEvent (Watched watched, string filename, string srcpath, EventType mask)
+		{
+			// Does the watch care about this event?
+			if ((watched.Mask & mask) == 0)
+				return;
+
+			if (Verbose) {
+				Console.WriteLine ("*** inotify: {0} {1} {2} {3} {4}",
+						   mask, watched.Wd, watched.Path,
+						   filename != "" ? filename : "\"\"",
+						   srcpath != null ? "(from " + srcpath + ")" : "");
+			}
+
+			if (Event != null) {
+				try {
+					Event (watched.Wd, watched.Path, filename, srcpath, mask);
+				} catch (Exception e) {
+					Logger.Log.Error ("Caught exception inside Inotify.Event");
+					Logger.Log.Error (e); 
+				}
+			}
+		}
+
 		static void DispatchWorker ()
 		{
 			Encoding filename_encoding = Encoding.UTF8;
@@ -399,8 +483,22 @@ namespace Beagle.Util {
 				queued_event qe;
 
 				lock (event_queue) {
-					while (event_queue.Count == 0 && running)
-						Monitor.Wait (event_queue);
+					while (event_queue.Count == 0 && running) {
+						// Find all unmatched MovedFrom events and process them.  We expire in
+						// five seconds.  This is quite conservative, but what is the rush?
+						// Would be nice to not do this here and not have to wake up every 2 secs
+						for (int i = 0; i < cookie_list.Count; i++) {
+							PendingMove pending = cookie_list[i] as PendingMove;
+							if (pending.Time.AddSeconds (2) < DateTime.Now) {
+								SendEvent (pending.Watch, pending.SrcName, null,
+									   EventType.MovedFrom);
+								cookie_hash.Remove (pending);
+								cookie_list.Remove (pending);
+							}
+						}
+
+						Monitor.Wait (event_queue, 2);			
+					}
 					if (!running)
 						break;
 					qe = (queued_event) event_queue.Dequeue ();
@@ -411,34 +509,61 @@ namespace Beagle.Util {
 				if (watched == null)
 					continue;
 
-				if ((watched.Mask & qe.iev.mask) != 0) {
-					
-					int n_chars = 0;
-					while (n_chars < qe.filename.Length && qe.filename [n_chars] != 0)
-						++n_chars;
+				// Get the filename payload, if any
+				int n_chars = 0;
+				while (n_chars < qe.filename.Length && qe.filename [n_chars] != 0)
+					++n_chars;
+				string filename = "";
+				if (n_chars > 0)
+					filename = filename_encoding.GetString (qe.filename, 0, n_chars);
 
-					string filename = "";
-					if (n_chars > 0)
-						filename = filename_encoding.GetString (qe.filename, 0, n_chars);
-
-					if (Verbose) {
-						Console.WriteLine ("*** inotify: {0} {1} {2} {3} {4}",
-								   qe.iev.mask, watched.Wd, watched.Path,
-								   filename != "" ? filename : "\"\"",
-								   qe.iev.cookie);
+				// If this is a directory, we need to handle MovedTo and MovedFrom events
+				// (regardless of the actual watched.Mask) and fix up watched.Path and the
+				// path of all subdirectories.
+				//
+				// We also have to handle unmatched events.  As we presume MovedTo always
+				// follows MovedFrom, we only need to handle unmatched MoveFrom's
+				// explicitly (unmatched MovedFrom's have srcpath=null in Inotify.Event).
+				// We handle the expiration process above, in the idle loop.
+				//
+				// Unfortunately, we have to handle the Directory.Exists() test a bit
+				// roundabout, because watched.Path during MovedFrom no longer exists.
+				if (qe.iev.mask == EventType.MovedFrom) {
+					PendingMove pending = new PendingMove (watched, filename, DateTime.Now,
+									       qe.iev.cookie);
+					cookie_hash [pending.Cookie] = pending;
+					cookie_list.Add (pending);
+					continue; // Wait for a possible matching MovedTo
+				}
+				string srcpath = null;
+				if (qe.iev.mask == EventType.MovedTo) {
+					PendingMove pending = cookie_hash [qe.iev.cookie] as PendingMove;
+					if (pending != null) {
+						string dstpath = Path.Combine (watched.Path, filename);
+						srcpath = Path.Combine (pending.Watch.Path, pending.SrcName);
+						if (Directory.Exists (dstpath))
+							HandleMove (srcpath, dstpath);
+						cookie_hash.Remove (pending);
+						cookie_list.Remove (pending);
 					}
 
-					if (Event != null) {
-						try {
-							Event (watched.Wd, watched.Path, filename, 
-							       qe.iev.mask, qe.iev.cookie);
-						} catch (Exception e) {
-							Logger.Log.Error ("Caught exception inside Inotify.Event");
-							Logger.Log.Error (e); 
+					// This gets prettier and prettier.  Like my Ex-Wife.  Since we delay
+					// unmatched MovedFrom events until they expire, a MovedFrom+MovedTo
+					// combination on the same file out and back into a watched directory
+					// can race and arrive as MovedTo+MovedFrom.  That is bad.
+					for (int i = 0; i < cookie_list.Count; i++) {
+						PendingMove race = cookie_list[i] as PendingMove;
+						if (race.Watch.Wd == watched.Wd && race.SrcName == filename) {
+							// Expire this MovedFrom and send it out right immediately
+							SendEvent (race.Watch, race.SrcName, null, EventType.MovedFrom);
+							cookie_hash.Remove (race);
+							cookie_list.Remove (race);
 						}
 					}
 				}
 
+				SendEvent (watched, filename, srcpath, qe.iev.mask);
+
 				// If a directory we are watching gets ignored, we need
 				// to remove it from the watchedByFoo hashes.
 				if (qe.iev.mask == EventType.Ignored) {
@@ -458,8 +583,11 @@ namespace Beagle.Util {
 			foreach (string arg in args) {
 				if (arg == "-r" || arg == "--recursive")
 					recursive = true;
-				else
-					to_watch.Enqueue (arg);
+				else {
+					// Our hashes work without a trailing path delimiter
+					string path = arg.TrimEnd ('/');
+					to_watch.Enqueue (path);
+				}
 			}
 
 			while (to_watch.Count > 0) {
