@@ -113,7 +113,7 @@ namespace Beagle.Daemon {
 							continue;
 
 						if (flavor.RequireInotify && ! Inotify.Enabled) {
-							Logger.Log.Debug ("Can't start backend '{0}' without inotify",  flavor.Name);
+							Logger.Log.Warn ("Can't start backend '{0}' without inotify",  flavor.Name);
 							continue;
 						}
 
