@@ -34,6 +34,8 @@ namespace Beagle.Daemon {
 
 	public class RemoteIndexer : IIndexer {
 
+		static public bool Debug = true;
+
 		static private TimeSpan one_second = new TimeSpan (10000000);
 
 		string remote_index_name;
@@ -58,11 +60,15 @@ namespace Beagle.Daemon {
 		private RemoteIndexerProxy Proxy {
 			get {
 				lock (this) {
-					if (proxy == null) {
+					if (proxy == null && ! Shutdown.ShutdownRequested) {
+						if (Debug)
+							Logger.Log.Debug ("Requesting new proxy '{0}'", remote_index_name);
 						proxy = IndexHelperFu.NewRemoteIndexerProxy (remote_index_name);
-						proxy.ChangedEvent += OnProxyChanged;
-						proxy.FlushCompleteEvent += OnFlushComplete;
-						add_remove_count = 0;
+						if (proxy != null) {
+							proxy.ChangedEvent += OnProxyChanged;
+							proxy.FlushCompleteEvent += OnFlushComplete;
+							add_remove_count = 0;
+						}
 					}
 					
 					return proxy;
@@ -74,6 +80,8 @@ namespace Beagle.Daemon {
 		{
 			lock (this) {
 				if (proxy != null) {
+					if (Debug)
+						Logger.Log.Debug ("Unsetting proxy '{0}'", remote_index_name);
 					proxy.ChangedEvent -= OnProxyChanged;
 					proxy.FlushCompleteEvent -= OnFlushComplete;
 					proxy = null;
@@ -88,6 +96,7 @@ namespace Beagle.Daemon {
 			RemoteIndexerProxy proxy;
 			int magic_code;
 			string data;
+			string data2;
 			bool finished;
 
 			static public UpToTheMainLoop NewAdd (RemoteIndexerProxy proxy, Indexable indexable)
@@ -104,7 +113,7 @@ namespace Beagle.Daemon {
 				UpToTheMainLoop up = new UpToTheMainLoop ();
 				up.proxy = proxy;
 				up.magic_code = 1;
-				up.data = uri.ToString ();
+				up.data = UriFu.UriToSerializableString (uri);
 				return up;
 			}
 
@@ -124,25 +133,44 @@ namespace Beagle.Daemon {
 				return up;
 			}
 
+			static public UpToTheMainLoop NewRename (RemoteIndexerProxy proxy, Uri old_uri, Uri new_uri)
+			{
+				UpToTheMainLoop up = new UpToTheMainLoop ();
+				up.proxy = proxy;
+				up.magic_code = 4;
+				up.data = UriFu.UriToSerializableString (old_uri);
+				up.data2 = UriFu.UriToSerializableString (new_uri);
+				return up;
+			}
+
 			private bool IdleHandler ()
 			{
-				if (Shutdown.ShutdownRequested)
-					return false;
-
 				lock (this) {
-					switch (magic_code) {
-					case 0:
-						proxy.Add (data);
-						break;
-					case 1:
-						proxy.Remove (data);
-						break;
-					case 2:
-						proxy.Flush ();
-						break;
-					case 3:
-						proxy.Close ();
-						break;
+					if (! Shutdown.ShutdownRequested) {
+
+						try {
+							switch (magic_code) {
+							case 0:
+								proxy.Add (data);
+								break;
+							case 1:
+								proxy.Remove (data);
+								break;
+							case 2:
+								proxy.Flush ();
+								break;
+							case 3:
+								proxy.Close ();
+								break;
+								
+							case 4:
+								proxy.Rename (data, data2);
+								break;
+							}
+						} catch (Exception ex) {
+							Logger.Log.Debug ("Caught exception in RemoteIndexer.IdleHandler");
+							Logger.Log.Debug (ex);
+						}
 					}
 
 					finished = true;
@@ -150,21 +178,34 @@ namespace Beagle.Daemon {
 				}
 				return false;
 			}
-			
-			public void Run ()
+
+			public bool Run ()
 			{
 				lock (this) {
+					// If the proxy is null, it means the index helper
+					// disconnected unexpectedly --- this can happen
+					// with scheduled events on daemon shutdown, so we
+					// just silently return.
+					if (proxy == null)
+						return false;
+
 					GLib.IdleHandler idle_handler = new GLib.IdleHandler (IdleHandler);
 					GLib.Idle.Add (idle_handler);
 					finished = false;
 					while (! finished) {
-						Logger.Log.Debug ("Waiting code={0}", magic_code);
-						Monitor.Wait (this, one_second);
+						if (Debug)
+							Logger.Log.Debug ("Waiting code={0}", magic_code);
 						// Bale out if a shutdown request has come in.
-						if (Shutdown.ShutdownRequested)
-							return;
+						if (Shutdown.ShutdownRequested) {
+							if (Debug)
+								Logger.Log.Debug ("Bailing out while waiting on code={0}", magic_code);
+							return false;
+						}
+						Monitor.Wait (this, one_second);
 					}
 				}
+
+				return true;
 			}
 			
 		}
@@ -176,32 +217,97 @@ namespace Beagle.Daemon {
 			
 #if DBUS_IS_BROKEN_BROKEN_BROKEN
 			UpToTheMainLoop up = UpToTheMainLoop.NewAdd (Proxy, indexable);
-			up.Run ();
+			if (up.Run ())
+				++add_remove_count;
 #else
-			Proxy.Add (indexable.ToString ());
+			RemoteIndexerProxy p = Proxy;
+			if (p != null) {
+				p.Add (indexable.ToString ());
+				++add_remove_count;
+			}
 #endif
-			++add_remove_count;
 		}
 
 		public void Remove (Uri uri)
 		{
 #if DBUS_IS_BROKEN_BROKEN_BROKEN
 			UpToTheMainLoop up = UpToTheMainLoop.NewRemove (Proxy, uri);
-			up.Run ();
+			if (up.Run ())
+				++add_remove_count;
 #else
-			Proxy.Remove (uri.ToString ());
+			RemoteIndexerProxy p = Proxy;
+			if (p != null) {
+				p.Remove (UriFu.UriToSerializableString (uri));
+				++add_remove_count;
+			}
 #endif
-			++add_remove_count;
 		}
 
+		public void Rename (Uri old_uri, Uri new_uri)
+		{
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+			UpToTheMainLoop up = UpToTheMainLoop.NewRename (Proxy, old_uri, new_uri);
+			if (up.Run ())
+				++add_remove_count;
+#else
+			RemoteIndexerProxy p = Proxy;
+			if (p != null) {
+				p.Rename (UriFu.UriToSerializableString (old_uri),
+					  UriFu.UriToSerializableString (new_uri));
+				++add_remove_count;
+			}
+#endif
+		}
+
+		// This can only be called from the main thread!
+		private void PostFlushComplete ()
+		{
+			RemoteIndexerProxy p = Proxy;
+			if (p != null) {
+				Logger.Log.Debug ("Calling close on proxy '{0}'", remote_index_name);
+				p.Close ();
+			}
+			
+			Logger.Log.Debug ("Unsetting '{0}'", remote_index_name);
+			UnsetProxy ();
+		}
+
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+
+		private uint flush_timeout_handler = 0;
+
+		private bool FlushTimeoutHandler ()
+		{
+			Logger.Log.Debug ("Checking status of FlushTimeoutHandler for '{0}'", remote_index_name);
+			// We know we are in main loop, so we can make any
+			// dbus call we want to.
+			if (! Proxy.IsFlushing ()) {
+				lock (flush_lock) {
+					if (! flush_complete) {
+						Logger.Log.Debug ("No longer flushing on '{0}', setting flush_complete", remote_index_name);
+						flush_complete = true;
+						PostFlushComplete ();
+					}
+					flush_timeout_handler = 0;
+				}
+				return false;
+			}
+			return true;
+		}
+#endif
+
 		public void Flush ()
 		{
+			Logger.Log.Debug ("RemoteIndexer.Flush on '{0}'", remote_index_name);
+
 			if (add_remove_count == 0) {
 #if DBUS_IS_BROKEN_BROKEN_BROKEN
 				UpToTheMainLoop up = UpToTheMainLoop.NewClose (Proxy);
 				up.Run ();
 #else
-				Proxy.Close ();
+				RemoteIndexerProxy p = Proxy;
+				if (p != null)
+					p.Close ();
 #endif
 				UnsetProxy ();
 				return;
@@ -212,46 +318,66 @@ namespace Beagle.Daemon {
 				UpToTheMainLoop up = UpToTheMainLoop.NewFlush (Proxy);
 				up.Run ();
 #else
-				Proxy.Flush ();
+				RemoteIndexerProxy p = Proxy;
+				p.Flush ();
 #endif
 				// Wait for the flush complete signal, but bail out
 				// if a shutdown request comes through.
 				flush_complete = false;
-				while (! flush_complete && ! Shutdown.ShutdownRequested) {
+				flush_timeout_handler = GLib.Timeout.Add (1000, new GLib.TimeoutHandler (FlushTimeoutHandler));
+				while (true) {
+
 					Logger.Log.Debug ("Waiting for flush to complete on '{0}'", remote_index_name);
-					Monitor.Wait (flush_lock, one_second);
+					if (Shutdown.ShutdownRequested) {
+						if (Debug)
+							Logger.Log.Debug ("Bailing out while waiting for flush on '{0}'", remote_index_name);
+						break;
+					}
+
+					lock (flush_lock) {
+						if (flush_complete)
+							break;
+						Monitor.Wait (flush_lock, one_second);
+						if (flush_complete)
+							break;
+					}
+				}
+				if (flush_timeout_handler != 0) {
+					GLib.Source.Remove (flush_timeout_handler);
+					flush_timeout_handler = 0;
 				}
-			
 			}
 		}
 
 		public int GetItemCount ()
 		{
-			return Proxy.GetItemCount ();
+			RemoteIndexerProxy p = Proxy;
+			return p != null ? p.GetItemCount () : -1;
 		}
 
-		private void OnProxyChanged (string list_of_added_uris_as_string,
-					     string list_of_removed_uris_as_string)
+		private void OnProxyChanged (string list_of_added_uris_as_str,
+					     string list_of_removed_uris_as_str,
+					     string list_of_renamed_uris_as_str)
 		{
 			if (ChangedEvent != null)
 				ChangedEvent (this,
-					      UriFu.StringToUris (list_of_added_uris_as_string),
-					      UriFu.StringToUris (list_of_removed_uris_as_string));
+					      UriFu.StringToUris (list_of_added_uris_as_str),
+					      UriFu.StringToUris (list_of_removed_uris_as_str),
+					      UriFu.StringToUris (list_of_renamed_uris_as_str));
 		}
 
 		private void OnFlushComplete ()
 		{
 			lock (flush_lock) {
-				Logger.Log.Debug ("Flush complete");
+				Logger.Log.Debug ("Got flush complete event from proxy '{0}'", remote_index_name);
+
+				if (! flush_complete)
+					PostFlushComplete ();
 				flush_complete = true;
 				
 				// Since this event is dispatched by d-bus, we are guaranteed
 				// to be in the main loop's thread.  Thus we don't have to
 				// jump through the same hoops as we did above.
-				Proxy.Close ();
-
-				UnsetProxy ();
-
 				Monitor.Pulse (flush_lock);
 			}
 		}
