@@ -120,8 +120,8 @@ namespace Beagle.Daemon.TomboyQueryable {
 		private void OnInotifyEvent (int wd,
 					     string path,
 					     string subitem,
-					     Inotify.EventType type,
-					     uint cookie)
+					     string srcpath,
+					     Inotify.EventType type)
 		{
 			if (wd != tomboy_wd)
 				return;
